@@ -8,10 +8,6 @@ module Brick.BChan
   )
 where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-#endif
-
 import Control.Concurrent.STM.TBQueue
 import Control.Monad.STM (atomically, orElse)
 
