@@ -1,7 +1,5 @@
 -- | This module provides a type and functions for handling focus rings
 -- of values.
---
--- This interface is experimental.
 module Brick.Focus
   ( FocusRing
   , focusRing
@@ -18,7 +16,7 @@ module Brick.Focus
 where
 
 import Lens.Micro ((^.))
-import Data.Maybe (listToMaybe)
+import Data.List (find)
 import qualified Data.CircularList as C
 
 import Brick.Types
@@ -60,13 +58,13 @@ withFocusRing :: (Eq n, Named a n)
               -> (Bool -> a -> b)
               -- ^ A function that takes a value and its focus state.
               -> a
-              -- ^ The wiget state value that we need to check for focus.
+              -- ^ The widget state value that we need to check for focus.
               -> b
               -- ^ The rest of the computation.
 withFocusRing ring f a = f (focusGetCurrent ring == Just (getName a)) a
 
 -- | Get the currently-focused resource name from the ring. If the ring
--- is emtpy, return 'Nothing'.
+-- is empty, return 'Nothing'.
 focusGetCurrent :: FocusRing n -> Maybe n
 focusGetCurrent (FocusRing l) = C.focus l
 
@@ -109,8 +107,5 @@ focusRingCursor :: (Eq n)
                 -> Maybe (CursorLocation n)
                 -- ^ The cursor position, if any, that matches the
                 -- resource name currently focused by the 'FocusRing'.
-focusRingCursor getRing st ls =
-    listToMaybe $ filter isCurrent ls
-    where
-        isCurrent cl = cl^.cursorLocationNameL ==
-                       (focusGetCurrent $ getRing st)
+focusRingCursor getRing st = find $ \cl ->
+    cl^.cursorLocationNameL == focusGetCurrent (getRing st)
