@@ -3,10 +3,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE ScopedTypeVariables #-}
--- | NOTE: This API is experimental and will probably change. Please try
--- it out! Feedback is very much appreciated, and your patience in the
--- face of breaking API changes is also appreciated! It's also worth
--- bearing in mind that this API is designed to support a narrow range
+-- | Note - this API is designed to support a narrow (but common!) set
 -- of use cases. If you find that you need more customization than this
 -- offers, then you will need to consider building your own layout and
 -- event handling for input fields.
@@ -32,6 +29,10 @@
 -- 'FormField's combined for a single 'FormFieldState' (e.g. a radio
 -- button sequence).
 --
+-- To use a 'Form', you must include it within your application state
+-- type. You can use 'formState' to access the underlying s whenever you
+-- need it. See @programs/FormDemo.hs@ for a complete working example.
+--
 -- Also note that, by default, forms and their field inputs are
 -- concatenated together in a 'vBox'. This can be customized on a
 -- per-field basis and for the entire form by using the functions
@@ -60,10 +61,12 @@ module Brick.Forms
   , setFormConcat
   , setFieldConcat
   , setFormFocus
+  , updateFormState
 
   -- * Simple form field constructors
   , editTextField
   , editShowableField
+  , editShowableFieldWithValidate
   , editPasswordField
   , radioField
   , checkboxField
@@ -85,7 +88,7 @@ import Graphics.Vty hiding (showCursor)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
 #endif
-import Data.Maybe (isJust, isNothing)
+import Data.Maybe (fromJust, isJust, isNothing)
 import Data.List (elemIndex)
 import Data.Vector (Vector)
 
@@ -118,12 +121,12 @@ data FormField a b e n =
               -- ^ A validation function converting this field's state
               -- into a value of your choosing. @Nothing@ indicates a
               -- validation failure. For example, this might validate
-              -- an 'Editor' state value by parsing its text contents s
-              -- aan integer and return 'Maybe' 'Int'. This is for pure
-              -- avalue validation; if additional validation is required
-              -- a(e.g. via 'IO'), use this field's state value in an
-              -- aexternal validation routine and use 'setFieldValid' to
-              -- afeed the result back into the form.
+              -- an 'Editor' state value by parsing its text contents as
+              -- an integer and return 'Maybe' 'Int'. This is for pure
+              -- value validation; if additional validation is required
+              -- (e.g. via 'IO'), use this field's state value in an
+              -- external validation routine and use 'setFieldValid' to
+              -- feed the result back into the form.
               , formFieldExternallyValid :: Bool
               -- ^ Whether the field is valid according to an external
               -- validation source. Defaults to always being 'True' and
@@ -169,6 +172,9 @@ data FormFieldState s e n where
                       -- ^ A lens to extract and store a
                       -- successfully-validated form input back into
                       -- your form state.
+                      , formFieldUpdate :: a -> b -> b
+                      -- ^ Given a new form state value, update the form
+                      -- field state in place.
                       , formFields :: [FormField a b e n]
                       -- ^ The form fields, in order, that the user will
                       -- interact with to manipulate this state value.
@@ -184,7 +190,8 @@ data FormFieldState s e n where
                       } -> FormFieldState s e n
 
 -- | A form: a sequence of input fields that manipulate the fields of an
--- underlying state that you choose.
+-- underlying state that you choose. This value must be stored in the
+-- Brick application's state.
 --
 -- Type variables are as follows:
 --
@@ -226,6 +233,25 @@ infixr 5 @@=
     let v = mkFs s
     in v { formFieldRenderHelper = h . (formFieldRenderHelper v) }
 
+-- | Update the state contained in a form.
+--
+-- This updates all form fields to be consistent with the new form
+-- state. Where possible, this attempts to maintain other input state,
+-- such as text editor cursor position.
+--
+-- Note that since this updates the form fields, this means that any
+-- field values will be completely overwritten! This may or may not
+-- be what you want, since a user actively using the form could get
+-- confused if their edits go away. Use carefully.
+updateFormState :: s -> Form s e n -> Form s e n
+updateFormState newState f =
+    let updateField fs = case fs of
+            FormFieldState st l upd s rh concatAll ->
+                FormFieldState (upd (newState^.l) st) l upd s rh concatAll
+    in f { formState = newState
+         , formFieldStates = updateField <$> formFieldStates f
+         }
+
 -- | Set the focused field of a form.
 setFormFocus :: (Eq n) => n -> Form s e n -> Form s e n
 setFormFocus n f = f { formFocus = focusSetCurrent n $ formFocus f }
@@ -257,7 +283,7 @@ newForm mkEs s =
             }
 
 formFieldNames :: FormFieldState s e n -> [n]
-formFieldNames (FormFieldState _ _ fields _ _) = formFieldName <$> fields
+formFieldNames (FormFieldState _ _ _ fields _ _) = formFieldName <$> fields
 
 -- | A form field for manipulating a boolean value. This represents
 -- 'True' as @[X] label@ and 'False' as @[ ] label@.
@@ -311,14 +337,16 @@ checkboxCustomField lb check rb stLens name label initialState =
                                                  handleEvent
                                      ]
                       , formFieldLens = stLens
+                      , formFieldUpdate =
+                          \val _ -> val
                       , formFieldRenderHelper = id
                       , formFieldConcat = vBox
                       }
 
-renderCheckbox :: Char -> Char -> Char -> T.Text -> n -> Bool -> Bool -> Widget n
+renderCheckbox :: (Ord n) => Char -> Char -> Char -> T.Text -> n -> Bool -> Bool -> Widget n
 renderCheckbox lb check rb label n foc val =
     let addAttr = if foc then withDefAttr focusedFormInputAttr else id
-        csr = if foc then showCursor n (Location (1,0)) else id
+        csr = if foc then putCursor n (Location (1,0)) else id
     in clickable n $
        addAttr $ csr $
        (txt $ T.singleton lb <> (if val then T.singleton check else " ") <>
@@ -365,6 +393,10 @@ listField options stLens renderItem itemHeight name initialState =
                                                  handleEvent
                                      ]
                       , formFieldLens = customStLens
+                      , formFieldUpdate = \listState l ->
+                           case listSelectedElement listState of
+                               Nothing -> l
+                               Just (_, e) -> listMoveToElement e l
                       , formFieldRenderHelper = id
                       , formFieldConcat = vBox
                       }
@@ -432,17 +464,18 @@ radioCustomField lb check rb stLens options initialState =
     in FormFieldState { formFieldState = initVal
                       , formFields = optionFields
                       , formFieldLens = stLens
+                      , formFieldUpdate = \val _ -> val
                       , formFieldRenderHelper = id
                       , formFieldConcat = vBox
                       }
 
-renderRadio :: (Eq a) => Char -> Char -> Char -> a -> n -> T.Text -> Bool -> a -> Widget n
+renderRadio :: (Eq a, Ord n) => Char -> Char -> Char -> a -> n -> T.Text -> Bool -> a -> Widget n
 renderRadio lb check rb val name label foc cur =
     let addAttr = if foc
                   then withDefAttr focusedFormInputAttr
                   else id
         isSet = val == cur
-        csr = if foc then showCursor name (Location (1,0)) else id
+        csr = if foc then putCursor name (Location (1,0)) else id
     in clickable name $
        addAttr $ csr $
        hBox [ txt $ T.singleton lb
@@ -468,7 +501,7 @@ editField :: (Ord n, Show n)
           -- the editor's initial contents. The resulting text may
           -- contain newlines.
           -> ([T.Text] -> Maybe a)
-          -- ^ The validation function that converts the editors
+          -- ^ The validation function that converts the editor's
           -- contents into a valid value of type @a@.
           -> ([T.Text] -> Widget n)
           -- ^ The rendering function for the editor's contents (see
@@ -499,6 +532,11 @@ editField stLens n limit ini val renderText wrapEditor initialState =
                                                  handleEvent
                                      ]
                       , formFieldLens = stLens
+                      , formFieldUpdate = \newVal e ->
+                          let newTxt = ini newVal
+                          in if newTxt == (T.unlines $ getEditContents e)
+                             then e
+                             else applyEdit (Z.insertMany newTxt . Z.clearZipper) e
                       , formFieldRenderHelper = id
                       , formFieldConcat = vBox
                       }
@@ -520,8 +558,36 @@ editShowableField :: (Ord n, Show n, Read a, Show a)
                   -- ^ The initial form state.
                   -> FormFieldState s e n
 editShowableField stLens n =
+    editShowableFieldWithValidate stLens n (const True)
+
+-- | A form field using a single-line editor to edit the 'Show' representation
+-- of a state field value of type @a@. This automatically uses its 'Read'
+-- instance to validate the input, and also accepts an additional user-defined
+-- pass for validation. This field is mostly useful in cases where the
+-- user-facing representation of a value matches the 'Show' representation
+-- exactly, such as with 'Int', but you don't want to accept just /any/ 'Int'.
+--
+-- This field responds to all events handled by 'editor', including
+-- mouse events.
+editShowableFieldWithValidate :: (Ord n, Show n, Read a, Show a)
+                              => Lens' s a
+                              -- ^ The state lens for this value.
+                              -> n
+                              -- ^ The resource name for the input field.
+                              -> (a -> Bool)
+                              -- ^ Additional validation step for input.
+                              -- 'True' indicates that the value is
+                              -- valid.
+                              -> s
+                              -- ^ The initial form state.
+                              -> FormFieldState s e n
+editShowableFieldWithValidate stLens n isValid =
     let ini = T.pack . show
-        val = readMaybe . T.unpack . T.intercalate "\n"
+        val ls = do
+            v <- readMaybe $ T.unpack $ T.intercalate "\n" ls
+            if isValid v
+               then return v
+               else Nothing
         limit = Just 1
         renderText = txt . T.unlines
     in editField stLens n limit ini val renderText id
@@ -613,18 +679,18 @@ setFieldValid v n form =
     let go1 [] = []
         go1 (s:ss) =
             let s' = case s of
-                       FormFieldState st l fs rh concatAll ->
+                       FormFieldState st l upd fs rh concatAll ->
                            let go2 [] = []
                                go2 (f@(FormField fn val _ r h):ff)
                                    | n == fn = FormField fn val v r h : ff
                                    | otherwise = f : go2 ff
-                           in FormFieldState st l (go2 fs) rh concatAll
+                           in FormFieldState st l upd (go2 fs) rh concatAll
             in s' : go1 ss
 
     in form { formFieldStates = go1 (formFieldStates form) }
 
 getInvalidFields :: FormFieldState s e n -> [n]
-getInvalidFields (FormFieldState st _ fs _ _) =
+getInvalidFields (FormFieldState st _ _ fs _ _) =
     let gather (FormField n validate extValid _ _) =
             if (not extValid || (isNothing $ validate st)) then [n] else []
     in concat $ gather <$> fs
@@ -654,7 +720,7 @@ renderFormFieldState :: (Eq n)
                      => FocusRing n
                      -> FormFieldState s e n
                      -> Widget n
-renderFormFieldState fr (FormFieldState st _ fields helper concatFields) =
+renderFormFieldState fr (FormFieldState st _ _ fields helper concatFields) =
     let renderFields [] = []
         renderFields ((FormField n validate extValid renderField _):fs) =
             let maybeInvalid = if (isJust $ validate st) && extValid
@@ -741,13 +807,13 @@ getFocusGrouping f n = findGroup (formFieldStates f)
 
 entryAfter :: (Eq a) => [a] -> a -> a
 entryAfter as a =
-    let Just i = elemIndex a as
+    let i = fromJust $ elemIndex a as
         i' = if i == length as - 1 then 0 else i + 1
     in as !! i'
 
 entryBefore :: (Eq a) => [a] -> a -> a
 entryBefore as a =
-    let Just i = elemIndex a as
+    let i = fromJust $ elemIndex a as
         i' = if i == 0 then length as - 1 else i - 1
     in as !! i'
 
@@ -763,7 +829,7 @@ handleFormFieldEvent n ev f = findFieldState [] (formFieldStates f)
         findFieldState _ [] = return f
         findFieldState prev (e:es) =
             case e of
-                FormFieldState st stLens fields helper concatAll -> do
+                FormFieldState st stLens upd fields helper concatAll -> do
                     let findField [] = return Nothing
                         findField (field:rest) =
                             case field of
@@ -780,7 +846,7 @@ handleFormFieldEvent n ev f = findFieldState [] (formFieldStates f)
                     case result of
                         Nothing -> findFieldState (prev <> [e]) es
                         Just (newSt, maybeSt) ->
-                            let newFieldState = FormFieldState newSt stLens fields helper concatAll
+                            let newFieldState = FormFieldState newSt stLens upd fields helper concatAll
                             in return $ f { formFieldStates = prev <> [newFieldState] <> es
                                           , formState = case maybeSt of
                                               Nothing -> formState f
