@@ -10,8 +10,10 @@ module Brick.Main
 
   -- * Event handler functions
   , continue
+  , continueWithoutRedraw
   , halt
   , suspendAndResume
+  , makeVisible
   , lookupViewport
   , lookupExtent
   , findClickedExtents
@@ -55,10 +57,6 @@ import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.State
 import Control.Monad.Trans.Reader
 import Control.Concurrent (forkIO, killThread)
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>))
-import Data.Monoid (mempty)
-#endif
 import qualified Data.Foldable as F
 import Data.Maybe (listToMaybe)
 import qualified Data.Map as M
@@ -75,25 +73,29 @@ import Graphics.Vty
   , nextEvent
   , mkVty
   , defaultConfig
+  , restoreInputState
+  , inputIface
   )
 import Graphics.Vty.Attributes (defAttr)
 
 import Brick.BChan (BChan, newBChan, readBChan, readBChan2, writeBChan)
-import Brick.Types (Widget, EventM(..))
+import Brick.Types (EventM(..))
 import Brick.Types.Internal
 import Brick.Widgets.Internal
 import Brick.AttrMap
 
 -- | The library application abstraction. Your application's operations
--- are represented here and passed to one of the various main functions
--- in this module. An application is in terms of an application state
--- type 's', an application event type 'e', and a resource name type
--- 'n'. In the simplest case 'e' is unused (left polymorphic or set to
--- '()'), but you may define your own event type and use 'customMain'
--- to provide custom events. The state type is the type of application
--- state to be provided by you and iteratively modified by event
--- handlers. The resource name type is the type of names you can assign
--- to rendering resources such as viewports and cursor locations.
+-- are provided in an @App@ and then the @App@ is provided to one of the
+-- various main functions in this module. An application @App s e n@
+-- is in terms of an application state type @s@, an application event
+-- type @e@, and a resource name type @n@. In the simplest case 'e' is
+-- unused (left polymorphic or set to @()@), but you may define your own
+-- event type and use 'customMain' to provide custom events. The state
+-- type @s@ is the type of application state to be provided by you and
+-- iteratively modified by event handlers. The resource name type @n@
+-- is the type of names you can assign to rendering resources such as
+-- viewports and cursor locations. Your application must define this
+-- type.
 data App s e n =
     App { appDraw :: s -> [Widget n]
         -- ^ This function turns your application state into a list of
@@ -110,7 +112,8 @@ data App s e n =
         -- ^ This function takes the current application state and an
         -- event and returns an action to be taken and a corresponding
         -- transformed application state. Possible options are
-        -- 'continue', 'suspendAndResume', and 'halt'.
+        -- 'continue', 'continueWithoutRedraw', 'suspendAndResume', and
+        -- 'halt'.
         , appStartEvent :: s -> EventM n s
         -- ^ This function gets called once just prior to the first
         -- drawing of your application. Here is where you can make
@@ -187,8 +190,11 @@ runWithVty vty brickChan mUserChan app initialRS initialSt = do
     let readEvent = case mUserChan of
           Nothing -> readBChan brickChan
           Just uc -> readBrickEvent brickChan uc
-        runInner rs st = do
-          (result, newRS) <- runVty vty readEvent app st (resetRenderState rs)
+        runInner rs es draw st = do
+          let nextRS = if draw
+                       then resetRenderState rs
+                       else rs
+          (result, newRS, newExtents) <- runVty vty readEvent app st nextRS es draw
           case result of
               SuspendAndResume act -> do
                   killThread pid
@@ -196,12 +202,18 @@ runWithVty vty brickChan mUserChan app initialRS initialSt = do
               Halt s -> do
                   killThread pid
                   return $ InternalHalt s
-              Continue s -> runInner newRS s
-    runInner initialRS initialSt
+              Continue s -> runInner newRS newExtents True s
+              ContinueWithoutRedraw s ->
+                  runInner newRS newExtents False s
+    runInner initialRS mempty True initialSt
 
 -- | The custom event loop entry point to use when the simpler ones
 -- don't permit enough control. Returns the final application state
 -- after the application halts.
+--
+-- Note that this function guarantees that the terminal input state
+-- prior to the first Vty initialization is the terminal input state
+-- that is restored on shutdown (regardless of exceptions).
 customMain :: (Ord n)
            => Vty
            -- ^ The initial Vty handle to use.
@@ -221,8 +233,13 @@ customMain :: (Ord n)
            -- ^ The initial application state.
            -> IO s
 customMain initialVty buildVty mUserChan app initialAppState = do
+    let restoreInitialState = restoreInputState $ inputIface initialVty
+
     (s, vty) <- customMainWithVty initialVty buildVty mUserChan app initialAppState
+        `E.catch` (\(e::E.SomeException) -> restoreInitialState >> E.throw e)
+
     shutdown vty
+    restoreInitialState
     return s
 
 -- | Like 'customMain', except the last 'Vty' handle used by the
@@ -259,12 +276,12 @@ customMainWithVty initialVty buildVty mUserChan app initialAppState = do
                     newVty <- buildVty
                     run newVty (newRS { renderCache = mempty }) newAppState brickChan
 
-    let emptyES = ES [] mempty
-        emptyRS = RS M.empty mempty S.empty mempty mempty
+    let emptyES = ES [] mempty mempty
+        emptyRS = RS M.empty mempty S.empty mempty mempty mempty
         eventRO = EventRO M.empty initialVty mempty emptyRS
 
     (st, eState) <- runStateT (runReaderT (runEventM (appStartEvent app initialAppState)) eventRO) emptyES
-    let initialRS = RS M.empty (esScrollRequests eState) S.empty mempty []
+    let initialRS = RS M.empty (esScrollRequests eState) S.empty mempty [] (requestedVisibleNames eState)
     brickChan <- newBChan 20
     run initialVty initialRS st brickChan
 
@@ -280,9 +297,14 @@ runVty :: (Ord n)
        -> App s e n
        -> s
        -> RenderState n
-       -> IO (Next s, RenderState n)
-runVty vty readEvent app appState rs = do
-    (firstRS, exts) <- renderApp vty app appState rs
+       -> [Extent n]
+       -> Bool
+       -> IO (Next s, RenderState n, [Extent n])
+runVty vty readEvent app appState rs prevExtents draw = do
+    (firstRS, exts) <- case draw of
+        True -> renderApp vty app appState rs
+        False -> return (rs, prevExtents)
+
     e <- readEvent
 
     (e', nextRS, nextExts) <- case e of
@@ -296,15 +318,15 @@ runVty vty readEvent app appState rs = do
         VtyEvent (EvMouseDown c r button mods) -> do
             let matching = findClickedExtents_ (c, r) exts
             case matching of
-                (Extent n (Location (ec, er)) _ (Location (oC, oR)):_) ->
+                (Extent n (Location (ec, er)) _:_) ->
                     -- If the clicked extent was registered as
                     -- clickable, send a click event. Otherwise, just
                     -- send the raw mouse event
                     case n `elem` firstRS^.clickableNamesL of
                         True -> do
                             let localCoords = Location (lc, lr)
-                                lc = c - ec + oC
-                                lr = r - er + oR
+                                lc = c - ec
+                                lr = r - er
 
                                 -- If the clicked extent was a viewport,
                                 -- adjust the local coordinates by
@@ -321,15 +343,15 @@ runVty vty readEvent app appState rs = do
         VtyEvent (EvMouseUp c r button) -> do
             let matching = findClickedExtents_ (c, r) exts
             case matching of
-                (Extent n (Location (ec, er)) _ (Location (oC, oR)):_) ->
+                (Extent n (Location (ec, er)) _:_) ->
                     -- If the clicked extent was registered as
                     -- clickable, send a click event. Otherwise, just
                     -- send the raw mouse event
                     case n `elem` firstRS^.clickableNamesL of
                         True -> do
                             let localCoords = Location (lc, lr)
-                                lc = c - ec + oC
-                                lr = r - er + oR
+                                lc = c - ec
+                                lr = r - er
                                 -- If the clicked extent was a viewport,
                                 -- adjust the local coordinates by
                                 -- adding the viewport upper-left corner
@@ -343,15 +365,19 @@ runVty vty readEvent app appState rs = do
                 _ -> return (e, firstRS, exts)
         _ -> return (e, firstRS, exts)
 
-    let emptyES = ES [] mempty
+    let emptyES = ES [] mempty mempty
         eventRO = EventRO (viewportMap nextRS) vty nextExts nextRS
 
     (next, eState) <- runStateT (runReaderT (runEventM (appHandleEvent app appState e'))
                                 eventRO) emptyES
-    return (next, nextRS { rsScrollRequests = esScrollRequests eState
-                         , renderCache = applyInvalidations (cacheInvalidateRequests eState) $
-                                         renderCache nextRS
-                         })
+    return ( next
+           , nextRS { rsScrollRequests = esScrollRequests eState
+                    , renderCache = applyInvalidations (cacheInvalidateRequests eState) $
+                                    renderCache nextRS
+                    , requestedVisibleNames_ = requestedVisibleNames eState
+                    }
+           , nextExts
+           )
 
 applyInvalidations :: (Ord n) => S.Set (CacheInvalidateRequest n) -> M.Map n v -> M.Map n v
 applyInvalidations ns cache =
@@ -366,14 +392,22 @@ applyInvalidations ns cache =
 -- information from the most recent rendering. Returns 'Nothing' if
 -- no such state could be found, either because the name was invalid
 -- or because no rendering has occurred (e.g. in an 'appStartEvent'
--- handler).
+-- handler). An important consequence of this behavior is that if this
+-- function is called before a viewport is rendered for the first
+-- time, no state will be found because the renderer only knows about
+-- viewports it has rendered in the most recent rendering. As a result,
+-- if you need to make viewport transformations before they are drawn
+-- for the first time, you may need to use 'viewportScroll' and its
+-- associated functions without relying on this function. Those
+-- functions queue up scrolling requests that can be made in advance of
+-- the next rendering to affect the viewport.
 lookupViewport :: (Ord n) => n -> EventM n (Maybe Viewport)
 lookupViewport n = EventM $ asks (M.lookup n . eventViewportMap)
 
 -- | Did the specified mouse coordinates (column, row) intersect the
 -- specified extent?
 clickedExtent :: (Int, Int) -> Extent n -> Bool
-clickedExtent (c, r) (Extent _ (Location (lc, lr)) (w, h) _) =
+clickedExtent (c, r) (Extent _ (Location (lc, lr)) (w, h)) =
    c >= lc && c < (lc + w) &&
    r >= lr && r < (lr + h)
 
@@ -382,7 +416,7 @@ clickedExtent (c, r) (Extent _ (Location (lc, lr)) (w, h) _) =
 lookupExtent :: (Eq n) => n -> EventM n (Maybe (Extent n))
 lookupExtent n = EventM $ asks (listToMaybe . filter f . latestExtents)
     where
-        f (Extent n' _ _ _) = n == n'
+        f (Extent n' _ _) = n == n'
 
 -- | Given a mouse click location, return the extents intersected by the
 -- click. The returned extents are sorted such that the first extent in
@@ -428,8 +462,11 @@ renderApp vty app appState rs = do
                                         rs
         picWithCursor = case theCursor of
             Nothing -> pic { picCursor = NoCursor }
-            Just cloc -> pic { picCursor = AbsoluteCursor (cloc^.locationColumnL)
-                                                          (cloc^.locationRowL)
+            Just cloc -> pic { picCursor = (if cursorLocationVisible cloc
+                                            then AbsoluteCursor
+                                            else PositionOnly True)
+                                           (cloc^.locationColumnL)
+                                           (cloc^.locationRowL)
                              }
 
     update vty picWithCursor
@@ -517,6 +554,17 @@ viewportScroll n =
 continue :: s -> EventM n (Next s)
 continue = return . Continue
 
+-- | Continue running the event loop with the specified application
+-- state without redrawing the screen. This is faster than 'continue'
+-- because it skips the redraw, but the drawback is that you need to
+-- be really sure that you don't want a screen redraw. If your state
+-- changed in a way that needs to be reflected on the screen, use
+-- 'continue'. This function is for cases where you know that you did
+-- something that won't have an impact on the screen state and you want
+-- to save on redraw cost.
+continueWithoutRedraw :: s -> EventM n (Next s)
+continueWithoutRedraw = return . ContinueWithoutRedraw
+
 -- | Halt the event loop and return the specified application state as
 -- the final state value.
 halt :: s -> EventM n (Next s)
@@ -526,5 +574,18 @@ halt = return . Halt
 -- specified action. When it returns an application state value, restore
 -- the terminal state, empty the rendering cache, redraw the application
 -- from the new state, and resume the event loop.
+--
+-- Note that any changes made to the terminal's input state are ignored
+-- when Brick resumes execution and are not preserved in the final
+-- terminal input state after the Brick application returns the terminal
+-- to the user.
 suspendAndResume :: IO s -> EventM n (Next s)
 suspendAndResume = return . SuspendAndResume
+
+-- | Request that the specified UI element be made visible on the
+-- next rendering. This is provided to allow event handlers to make
+-- visibility requests in the same way that the 'visible' function does
+-- at rendering time.
+makeVisible :: (Ord n) => n -> EventM n ()
+makeVisible n = EventM $ do
+    lift $ modify (\s -> s { requestedVisibleNames = S.insert n $ requestedVisibleNames s })
