@@ -3,7 +3,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TupleSections #-}
--- | This module provids a file browser widget that allows users to
+-- | This module provides a file browser widget that allows users to
 -- navigate directory trees, search for files and directories, and
 -- select entries of interest. For a complete working demonstration of
 -- this module, see @programs/FileBrowserDemo.hs@.
@@ -65,8 +65,22 @@ module Brick.Widgets.FileBrowser
   , updateFileBrowserSearch
   , setFileBrowserEntryFilter
 
+  -- * Actions
+  , actionFileBrowserBeginSearch
+  , actionFileBrowserSelectEnter
+  , actionFileBrowserSelectCurrent
+  , actionFileBrowserListPageUp
+  , actionFileBrowserListPageDown
+  , actionFileBrowserListHalfPageUp
+  , actionFileBrowserListHalfPageDown
+  , actionFileBrowserListTop
+  , actionFileBrowserListBottom
+  , actionFileBrowserListNext
+  , actionFileBrowserListPrev
+
   -- * Handling events
   , handleFileBrowserEvent
+  , maybeSelectCurrentEntry
 
   -- * Rendering
   , renderFileBrowser
@@ -97,7 +111,6 @@ module Brick.Widgets.FileBrowser
   , fileExtensionMatch
 
   -- * Lenses
-  , fileBrowserEntryFilterL
   , fileBrowserSelectableL
   , fileInfoFilenameL
   , fileInfoSanitizedFilenameL
@@ -107,6 +120,17 @@ module Brick.Widgets.FileBrowser
   , fileStatusSizeL
   , fileStatusFileTypeL
 
+  -- * Getters
+  , fileBrowserEntryFilterG
+  , fileBrowserWorkingDirectoryG
+  , fileBrowserEntriesG
+  , fileBrowserLatestResultsG
+  , fileBrowserSelectedFilesG
+  , fileBrowserNameG
+  , fileBrowserSearchStringG
+  , fileBrowserExceptionG
+  , fileBrowserSelectableG
+
   -- * Miscellaneous
   , prettyFileSize
 
@@ -131,6 +155,7 @@ import Data.List (sortBy, isSuffixOf)
 import qualified Data.Set as Set
 import qualified Data.Vector as V
 import Lens.Micro
+import Lens.Micro.TH (lensRules, generateUpdateableOptics)
 import qualified Graphics.Vty as Vty
 import qualified System.Directory as D
 import qualified System.Posix.Files as U
@@ -224,6 +249,7 @@ data FileType =
     deriving (Read, Show, Eq)
 
 suffixLenses ''FileBrowser
+suffixLensesWith "G" (lensRules & generateUpdateableOptics .~ False) ''FileBrowser
 suffixLenses ''FileInfo
 suffixLenses ''FileStatus
 
@@ -297,6 +323,12 @@ selectDirectories i =
 -- indicates no filtering, meaning all entries will be shown. 'Just'
 -- indicates a function that should return 'True' for entries that
 -- should be permitted to appear.
+--
+-- Note that this applies the filter after setting it by updating the
+-- listed entries to reflect the result of the filter. That is unlike
+-- setting the filter with the 'fileBrowserEntryFilterL' lens directly,
+-- which just sets the filter but does not (and cannot) update the
+-- listed entries.
 setFileBrowserEntryFilter :: Maybe (FileInfo -> Bool) -> FileBrowser n -> FileBrowser n
 setFileBrowserEntryFilter f b =
     applyFilterAndSearch $ b & fileBrowserEntryFilterL .~ f
@@ -539,21 +571,81 @@ fileBrowserCursor b = snd <$> listSelectedElement (b^.fileBrowserEntriesL)
 --
 -- Events handled regardless of mode:
 --
--- * @Enter@, @Space@: set the file browser's selected entry
---   ('fileBrowserSelection') for use by the calling application,
---   subject to 'fileBrowserSelectable'.
--- * @Ctrl-n@: select the next entry
--- * @Ctrl-p@: select the previous entry
--- * 'List' navigation keys
+-- * @Ctrl-b@: 'actionFileBrowserListPageUp'
+-- * @Ctrl-f@: 'actionFileBrowserListPageDown'
+-- * @Ctrl-d@: 'actionFileBrowserListHalfPageDown'
+-- * @Ctrl-u@: 'actionFileBrowserListHalfPageUp'
+-- * @g@: 'actionFileBrowserListTop'
+-- * @G@: 'actionFileBrowserListBottom'
+-- * @j@: 'actionFileBrowserListNext'
+-- * @k@: 'actionFileBrowserListPrev'
+-- * @Ctrl-n@: 'actionFileBrowserListNext'
+-- * @Ctrl-p@: 'actionFileBrowserListPrev'
 --
 -- Events handled only in normal mode:
 --
--- * @/@: enter search mode
+-- * @/@: 'actionFileBrowserBeginSearch'
+-- * @Enter@: 'actionFileBrowserSelectEnter'
+-- * @Space@: 'actionFileBrowserSelectCurrent'
 --
 -- Events handled only in search mode:
 --
 -- * @Esc@, @Ctrl-C@: cancel search mode
 -- * Text input: update search string
+
+actionFileBrowserBeginSearch :: FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserBeginSearch b =
+    return $ updateFileBrowserSearch (const $ Just "") b
+
+actionFileBrowserSelectEnter :: FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserSelectEnter b =
+    maybeSelectCurrentEntry b
+
+actionFileBrowserSelectCurrent :: FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserSelectCurrent b =
+    selectCurrentEntry b
+
+actionFileBrowserListPageUp :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListPageUp b = do
+    let old = b ^. fileBrowserEntriesL
+    new <- listMovePageUp old
+    return $ b & fileBrowserEntriesL .~ new
+
+actionFileBrowserListPageDown :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListPageDown b = do
+    let old = b ^. fileBrowserEntriesL
+    new <- listMovePageDown old
+    return $ b & fileBrowserEntriesL .~ new
+
+actionFileBrowserListHalfPageUp :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListHalfPageUp b = do
+    let old = b ^. fileBrowserEntriesL
+    new <- listMoveByPages (-0.5::Double) old
+    return $ b & fileBrowserEntriesL .~ new
+
+actionFileBrowserListHalfPageDown :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListHalfPageDown b = do
+    let old = b ^. fileBrowserEntriesL
+    new <- listMoveByPages (0.5::Double) old
+    return $ b & fileBrowserEntriesL .~ new
+
+actionFileBrowserListTop :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListTop b =
+    return $ b & fileBrowserEntriesL %~ listMoveTo 0
+
+actionFileBrowserListBottom :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListBottom b = do
+    let sz = length (listElements $ b^.fileBrowserEntriesL)
+    return $ b & fileBrowserEntriesL %~ listMoveTo (sz - 1)
+
+actionFileBrowserListNext :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListNext b =
+    return $ b & fileBrowserEntriesL %~ listMoveBy 1
+
+actionFileBrowserListPrev :: Ord n => FileBrowser n -> EventM n (FileBrowser n)
+actionFileBrowserListPrev b =
+    return $ b & fileBrowserEntriesL %~ listMoveBy (-1)
+
 handleFileBrowserEvent :: (Ord n) => Vty.Event -> FileBrowser n -> EventM n (FileBrowser n)
 handleFileBrowserEvent e b =
     if fileBrowserIsSearching b
@@ -586,51 +678,48 @@ handleFileBrowserEventNormal e b =
     case e of
         Vty.EvKey (Vty.KChar '/') [] ->
             -- Begin file search
-            return $ updateFileBrowserSearch (const $ Just "") b
+            actionFileBrowserBeginSearch b
         Vty.EvKey Vty.KEnter [] ->
             -- Select file or enter directory
-            maybeSelectCurrentEntry b
+            actionFileBrowserSelectEnter b
         Vty.EvKey (Vty.KChar ' ') [] ->
             -- Select entry
-            selectCurrentEntry b
+            actionFileBrowserSelectCurrent b
         _ ->
             handleFileBrowserEventCommon e b
 
 handleFileBrowserEventCommon :: (Ord n) => Vty.Event -> FileBrowser n -> EventM n (FileBrowser n)
 handleFileBrowserEventCommon e b =
     case e of
-        Vty.EvKey (Vty.KChar 'b') [Vty.MCtrl] -> do
-            let old = b ^. fileBrowserEntriesL
-            new <- listMovePageUp old
-            return $ b & fileBrowserEntriesL .~ new
-        Vty.EvKey (Vty.KChar 'f') [Vty.MCtrl] -> do
-            let old = b ^. fileBrowserEntriesL
-            new <- listMovePageDown old
-            return $ b & fileBrowserEntriesL .~ new
-        Vty.EvKey (Vty.KChar 'd') [Vty.MCtrl] -> do
-            let old = b ^. fileBrowserEntriesL
-            new <- listMoveByPages (0.5::Double) old
-            return $ b & fileBrowserEntriesL .~ new
-        Vty.EvKey (Vty.KChar 'u') [Vty.MCtrl] -> do
-            let old = b ^. fileBrowserEntriesL
-            new <- listMoveByPages (-0.5::Double) old
-            return $ b & fileBrowserEntriesL .~ new
+        Vty.EvKey (Vty.KChar 'b') [Vty.MCtrl] ->
+            actionFileBrowserListPageUp b
+        Vty.EvKey (Vty.KChar 'f') [Vty.MCtrl] ->
+            actionFileBrowserListPageDown b
+        Vty.EvKey (Vty.KChar 'd') [Vty.MCtrl] ->
+            actionFileBrowserListHalfPageDown b
+        Vty.EvKey (Vty.KChar 'u') [Vty.MCtrl] ->
+            actionFileBrowserListHalfPageUp b
         Vty.EvKey (Vty.KChar 'g') [] ->
-            return $ b & fileBrowserEntriesL %~ listMoveTo 0
-        Vty.EvKey (Vty.KChar 'G') [] -> do
-            let sz = length (listElements $ b^.fileBrowserEntriesL)
-            return $ b & fileBrowserEntriesL %~ listMoveTo (sz - 1)
+            actionFileBrowserListTop b
+        Vty.EvKey (Vty.KChar 'G') [] ->
+            actionFileBrowserListBottom b
         Vty.EvKey (Vty.KChar 'j') [] ->
-            return $ b & fileBrowserEntriesL %~ listMoveBy 1
+            actionFileBrowserListNext b
         Vty.EvKey (Vty.KChar 'k') [] ->
-            return $ b & fileBrowserEntriesL %~ listMoveBy (-1)
+            actionFileBrowserListPrev b
         Vty.EvKey (Vty.KChar 'n') [Vty.MCtrl] ->
-            return $ b & fileBrowserEntriesL %~ listMoveBy 1
+            actionFileBrowserListNext b
         Vty.EvKey (Vty.KChar 'p') [Vty.MCtrl] ->
-            return $ b & fileBrowserEntriesL %~ listMoveBy (-1)
+            actionFileBrowserListPrev b
         _ ->
             handleEventLensed b fileBrowserEntriesL handleListEvent e
 
+-- | If the browser's current entry is selectable according to
+-- @fileBrowserSelectable@, add it to the selection set and return.
+-- If not, and if the entry is a directory or a symlink targeting a
+-- directory, set the browser's current path to the selected directory.
+--
+-- Otherwise, return the browser state unchanged.
 maybeSelectCurrentEntry :: FileBrowser n -> EventM n (FileBrowser n)
 maybeSelectCurrentEntry b =
     case fileBrowserCursor b of
@@ -708,14 +797,14 @@ renderFileBrowser foc b =
 
     in withDefAttr fileBrowserAttr $
        vBox [ withDefAttr fileBrowserCurrentDirectoryAttr cwdHeader
-            , renderList (renderFileInfo foc maxFilenameLength (b^.fileBrowserSelectedFilesL))
+            , renderList (renderFileInfo foc maxFilenameLength (b^.fileBrowserSelectedFilesL) (b^.fileBrowserNameL))
                          foc (b^.fileBrowserEntriesL)
             , maybeSearchInfo
             , withDefAttr fileBrowserSelectionInfoAttr selInfo
             ]
 
-renderFileInfo :: Bool -> Int -> Set.Set String -> Bool -> FileInfo -> Widget n
-renderFileInfo foc maxLen selFiles listSel info =
+renderFileInfo :: Bool -> Int -> Set.Set String -> n -> Bool -> FileInfo -> Widget n
+renderFileInfo foc maxLen selFiles n listSel info =
     (if foc
      then (if listSel then forceAttr listSelectedFocusedAttr
                else if sel then forceAttr fileBrowserSelectedAttr else id)
@@ -727,6 +816,7 @@ renderFileInfo foc maxLen selFiles listSel info =
         addAttr = maybe id (withDefAttr . attrForFileType) (fileInfoFileType info)
         body = addAttr (hLimit (maxLen + 1) $
                padRight Max $
+               (if foc && listSel then putCursor n (Location (0,0)) else id) $
                str $ fileInfoSanitizedFilename info <> suffix)
         suffix = (if fileInfoFileType info == Just Directory then "/" else "") <>
                  (if sel then "*" else "")
