@@ -7,10 +7,6 @@ module Brick.Widgets.Internal
   )
 where
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative
-#endif
-
 import Lens.Micro ((^.), (&), (%~))
 import Control.Monad.Trans.State.Lazy
 import Control.Monad.Trans.Reader
@@ -30,16 +26,36 @@ renderFinal :: AttrMap
             -> ([CursorLocation n] -> Maybe (CursorLocation n))
             -> RenderState n
             -> (RenderState n, V.Picture, Maybe (CursorLocation n), [Extent n])
-renderFinal aMap layerRenders sz chooseCursor rs = (newRS, picWithBg, theCursor, concat layerExtents)
+renderFinal aMap layerRenders (w, h) chooseCursor rs =
+    (newRS, picWithBg, theCursor, concat layerExtents)
     where
         (layerResults, !newRS) = flip runState rs $ sequence $
             (\p -> runReaderT p ctx) <$>
             (render <$> cropToContext <$> layerRenders)
-        ctx = Context mempty (fst sz) (snd sz) defaultBorderStyle aMap False
-        pic = V.picForLayers $ uncurry V.resize sz <$> (^.imageL) <$> layerResults
+
+        ctx = Context { ctxAttrName = mempty
+                      , availWidth = w
+                      , availHeight = h
+                      , windowWidth = w
+                      , windowHeight = h
+                      , ctxBorderStyle = defaultBorderStyle
+                      , ctxAttrMap = aMap
+                      , ctxDynBorders = False
+                      , ctxVScrollBarOrientation = Nothing
+                      , ctxVScrollBarRenderer = Nothing
+                      , ctxHScrollBarOrientation = Nothing
+                      , ctxHScrollBarRenderer = Nothing
+                      , ctxHScrollBarShowHandles = False
+                      , ctxVScrollBarShowHandles = False
+                      , ctxHScrollBarClickableConstr = Nothing
+                      , ctxVScrollBarClickableConstr = Nothing
+                      }
+        pic = V.picForLayers $ uncurry V.resize (w, h) <$> (^.imageL) <$> layerResults
+
         -- picWithBg is a workaround for runaway attributes.
         -- See https://github.com/coreyoconnor/vty/issues/95
         picWithBg = pic { V.picBackground = V.Background ' ' V.defAttr }
+
         layerCursors = (^.cursorsL) <$> layerResults
         layerExtents = reverse $ (^.extentsL) <$> layerResults
         theCursor = chooseCursor $ concat layerCursors
@@ -58,10 +74,10 @@ cropResultToContext result = do
                     & extentsL %~ cropExtents c
                     & bordersL %~ cropBorders c
 
-cropImage :: Context -> V.Image -> V.Image
+cropImage :: Context n -> V.Image -> V.Image
 cropImage c = V.crop (max 0 $ c^.availWidthL) (max 0 $ c^.availHeightL)
 
-cropCursors :: Context -> [CursorLocation n] -> [CursorLocation n]
+cropCursors :: Context n -> [CursorLocation n] -> [CursorLocation n]
 cropCursors ctx cs = catMaybes $ cropCursor <$> cs
     where
         -- A cursor location is removed if it is not within the region
@@ -75,7 +91,7 @@ cropCursors ctx cs = catMaybes $ cropCursor <$> cs
                , c^.cursorLocationL.locationColumnL >= ctx^.availWidthL
                ]
 
-cropExtents :: Context -> [Extent n] -> [Extent n]
+cropExtents :: Context n -> [Extent n] -> [Extent n]
 cropExtents ctx es = catMaybes $ cropExtent <$> es
     where
         -- An extent is cropped in places where it is not within the
@@ -83,34 +99,26 @@ cropExtents ctx es = catMaybes $ cropExtent <$> es
         --
         -- If its entirety is outside the context region, it is dropped.
         --
-        -- Otherwise its size and upper left corner are adjusted so that
-        -- they are contained within the context region.
-        cropExtent (Extent n (Location (c, r)) (w, h) (Location (oC, oR))) =
-            -- First, clamp the upper-left corner to at least (0, 0).
-            let c' = max c 0
-                r' = max r 0
-                -- Compute deltas for the offset since if the upper-left
-                -- corner moved, so should the offset.
-                dc = c' - c
-                dr = r' - r
-                -- Then, determine the new lower-right corner based on
-                -- the clamped corner.
-                endCol = c' + w
-                endRow = r' + h
+        -- Otherwise its size is adjusted so that it is contained within
+        -- the context region.
+        cropExtent (Extent n (Location (c, r)) (w, h)) =
+            -- Determine the new lower-right corner
+            let endCol = c + w
+                endRow = r + h
                 -- Then clamp the lower-right corner based on the
                 -- context
                 endCol' = min (ctx^.availWidthL) endCol
                 endRow' = min (ctx^.availHeightL) endRow
                 -- Then compute the new width and height from the
                 -- clamped lower-right corner.
-                w' = endCol' - c'
-                h' = endRow' - r'
-                e = Extent n (Location (c', r')) (w', h') (Location (oC + dc, oR + dr))
+                w' = endCol' - c
+                h' = endRow' - r
+                e = Extent n (Location (c, r)) (w', h')
             in if w' < 0 || h' < 0
                then Nothing
                else Just e
 
-cropBorders :: Context -> BorderMap DynBorder -> BorderMap DynBorder
+cropBorders :: Context n -> BorderMap DynBorder -> BorderMap DynBorder
 cropBorders ctx = BM.crop Edges
     { eTop = 0
     , eBottom = availHeight ctx - 1
@@ -119,17 +127,19 @@ cropBorders ctx = BM.crop Edges
     }
 
 renderDynBorder :: DynBorder -> V.Image
-renderDynBorder db = V.char (dbAttr db) . ($dbStyle db) $ case bsDraw <$> dbSegments db of
-    --    top   bot   left  right
-    Edges False False False False -> const ' ' -- dunno lol (but should never happen, so who cares)
-    Edges False False _     _     -> bsHorizontal
-    Edges _     _     False False -> bsVertical
-    Edges False True  False True  -> bsCornerTL
-    Edges False True  True  False -> bsCornerTR
-    Edges True  False False True  -> bsCornerBL
-    Edges True  False True  False -> bsCornerBR
-    Edges False True  True  True  -> bsIntersectT
-    Edges True  False True  True  -> bsIntersectB
-    Edges True  True  False True  -> bsIntersectL
-    Edges True  True  True  False -> bsIntersectR
-    Edges True  True  True  True  -> bsIntersectFull
+renderDynBorder db = V.char (dbAttr db) $ getBorderChar $ dbStyle db
+    where
+        getBorderChar = case bsDraw <$> dbSegments db of
+            --    top   bot   left  right
+            Edges False False False False -> const ' '
+            Edges False False _     _     -> bsHorizontal
+            Edges _     _     False False -> bsVertical
+            Edges False True  False True  -> bsCornerTL
+            Edges False True  True  False -> bsCornerTR
+            Edges True  False False True  -> bsCornerBL
+            Edges True  False True  False -> bsCornerBR
+            Edges False True  True  True  -> bsIntersectT
+            Edges True  False True  True  -> bsIntersectB
+            Edges True  True  False True  -> bsIntersectL
+            Edges True  True  True  False -> bsIntersectR
+            Edges True  True  True  True  -> bsIntersectFull
