@@ -55,6 +55,8 @@ module Brick.Widgets.List
   , listMoveByPages
   , listMovePageUp
   , listMovePageDown
+  , listMoveToBeginning
+  , listMoveToEnd
   , listInsert
   , listRemove
   , listReplace
@@ -75,20 +77,18 @@ where
 
 import Prelude hiding (reverse, splitAt)
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative ((<$>), (<*>), pure)
-import Data.Foldable (Foldable, find, toList)
-import Data.Traversable (Traversable)
-#else
+import Control.Applicative ((<|>))
 import Data.Foldable (find, toList)
-#endif
 import Control.Monad.Trans.State (evalState, get, put)
 
 import Lens.Micro ((^.), (^?), (&), (.~), (%~), _2, _head, set)
 import Data.Functor (($>))
 import Data.List.NonEmpty (NonEmpty((:|)))
 import Data.Maybe (fromMaybe)
-import Data.Semigroup (Semigroup, (<>), sconcat)
+#if !(MIN_VERSION_base(4,11,0))
+import Data.Semigroup (Semigroup, (<>))
+#endif
+import Data.Semigroup (sconcat)
 import qualified Data.Sequence as Seq
 import Graphics.Vty (Event(..), Key(..), Modifier(..))
 import qualified Data.Vector as V
@@ -199,8 +199,8 @@ handleListEvent e theList =
     case e of
         EvKey KUp [] -> return $ listMoveUp theList
         EvKey KDown [] -> return $ listMoveDown theList
-        EvKey KHome [] -> return $ listMoveTo 0 theList
-        EvKey KEnd [] -> return $ listMoveTo (length $ listElements theList) theList
+        EvKey KHome [] -> return $ listMoveToBeginning theList
+        EvKey KEnd [] -> return $ listMoveToEnd theList
         EvKey KPageDown [] -> listMovePageDown theList
         EvKey KPageUp [] -> listMovePageUp theList
         _ -> return theList
@@ -229,14 +229,26 @@ handleListEventVi fallback e theList =
     case e of
         EvKey (KChar 'k') [] -> return $ listMoveUp theList
         EvKey (KChar 'j') [] -> return $ listMoveDown theList
-        EvKey (KChar 'g') [] -> return $ listMoveTo 0 theList
-        EvKey (KChar 'G') [] -> return $ listMoveTo (length $ listElements theList) theList
+        EvKey (KChar 'g') [] -> return $ listMoveToBeginning theList
+        EvKey (KChar 'G') [] -> return $ listMoveToEnd theList
         EvKey (KChar 'f') [MCtrl] -> listMovePageDown theList
         EvKey (KChar 'b') [MCtrl] -> listMovePageUp theList
         EvKey (KChar 'd') [MCtrl] -> listMoveByPages (0.5::Double) theList
         EvKey (KChar 'u') [MCtrl] -> listMoveByPages (-0.5::Double) theList
         _ -> fallback e theList
 
+-- | Move the list selection to the first element in the list.
+listMoveToBeginning :: (Foldable t, Splittable t)
+                    => GenericList n t e
+                    -> GenericList n t e
+listMoveToBeginning = listMoveTo 0
+
+-- | Move the list selection to the last element in the list.
+listMoveToEnd :: (Foldable t, Splittable t)
+              => GenericList n t e
+              -> GenericList n t e
+listMoveToEnd l = listMoveTo (max 0 $ length (listElements l) - 1) l
+
 -- | The top-level attribute used for the entire list.
 listAttr :: AttrName
 listAttr = "list"
@@ -527,7 +539,7 @@ listMoveBy amt l =
 -- If @pos >= 0@, indexes from the start of the list (which gets
 -- evaluated up to the target index)
 --
--- If @pos < 0@, indexes from the end of the list (which evalutes
+-- If @pos < 0@, indexes from the end of the list (which evaluates
 -- 'length' of the list).
 --
 -- Complexity: same as 'splitAt' for the container type.
@@ -571,10 +583,11 @@ listMoveToElement :: (Eq e, Foldable t, Splittable t)
                   -> GenericList n t e
 listMoveToElement e = listFindBy (== e) . set listSelectedL Nothing
 
--- | Set the selected index to the next element matching the
--- predicate.  If there is no selected element, the search starts at
--- the beginning.  If no matching element is found, leave the list
--- unmodified.
+-- | Starting from the currently-selected position, attempt to find
+-- and select the next element matching the predicate. If there are no
+-- matches for the remainder of the list or if the list has no selection
+-- at all, the search starts at the beginning. If no matching element is
+-- found anywhere in the list, leave the list unmodified.
 --
 -- /O(n)/.  Only evaluates as much of the container as needed.
 listFindBy :: (Foldable t, Splittable t)
@@ -583,9 +596,11 @@ listFindBy :: (Foldable t, Splittable t)
            -> GenericList n t e
 listFindBy test l =
     let start = maybe 0 (+1) (l ^. listSelectedL)
-        (_, t) = splitAt start (l ^. listElementsL)
-        result = find (test . snd) . zip [0..] . toList $ t
-    in maybe id (set listSelectedL . Just . (start +) . fst) result l
+        (h, t) = splitAt start (l ^. listElementsL)
+        tailResult = find (test . snd) . zip [start..] . toList $ t
+        headResult = find (test . snd) . zip [0..] . toList $ h
+        result = tailResult <|> headResult
+    in maybe id (set listSelectedL . Just . fst) result l
 
 -- | Return a list's selected element, if any.
 --
